#!/usr/bin/env node
var minimist = require('minimist');
var extend = require('xtend');
var typings_1 = require('../typings');
var cli_1 = require('../utils/cli');
var config_1 = require('../utils/config');
var registry_1 = require('../lib/registry');
var cli_2 = require('../utils/cli');
var args = minimist(process.argv.slice(2), {
    boolean: ['save', 'saveDev', 'ambient', 'verbose', 'help', 'production'],
    string: ['name', 'source'],
    alias: {
        save: ['S'],
        saveDev: ['save-dev', 'D'],
        name: ['n'],
        ambient: ['A'],
        verbose: ['v'],
        help: ['h'],
        source: ['s']
    }
});
if (args.help) {
    console.log("\n" + config_1.PROJECT_NAME + " install (with no arguments, in package directory)\n" + config_1.PROJECT_NAME + " install <pkg>[@<version>] --source [" + registry_1.VALID_SOURCES.join(' | ') + "]\n" + config_1.PROJECT_NAME + " install file:<path>\n" + config_1.PROJECT_NAME + " install github:<github username>/<github project>[/<path>][#<commit>]\n" + config_1.PROJECT_NAME + " install bitbucket:<bitbucket username>/<bitbucket project>[/<path>][#<commit>]\n" + config_1.PROJECT_NAME + " install <http:// url>\n\nAliases: i, in\nOptions: [--save|--save-dev] [--ambient] [--production]\n");
    process.exit(0);
}
function installer(args) {
    var options = extend(args, { cwd: process.cwd() });
    if (!args._.length) {
        return cli_1.loader(typings_1.install(options), args)
            .then(function (tree) {
            console.log(cli_2.archifyDependencyTree(tree, { ambient: true, dev: true }));
        });
    }
    var dependency = args._[0];
    var source = args.source || 'npm';
    if (!registry_1.isRegistryPath(dependency)) {
        return cli_1.loader(typings_1.installDependency(dependency, options), args)
            .then(function (tree) {
            console.log(cli_2.archifyDependencyTree(tree, { name: options.name }));
        });
    }
    var _a = registry_1.parseRegistryPath(dependency), name = _a.name, version = _a.version;
    return cli_1.loader(registry_1.read({ name: name, version: version, source: source }), args)
        .then(function (locations) {
        if (locations.length === 1) {
            return locations[0];
        }
        return cli_1.inquire([{
                name: 'location',
                type: 'list',
                message: 'Select the type definition to install',
                choices: locations
            }])
            .then(function (answers) { return answers.location; });
    })
        .then(function (location) {
        var installation = typings_1.installDependency(location, extend({ name: name }, options));
        return cli_1.loader(installation, args);
    })
        .then(function (tree) {
        console.log(cli_2.archifyDependencyTree(tree, { name: name }));
    });
}
installer(args);
//# sourceMappingURL=typings-install.js.map