var invariant = require('invariant');
var semver = require('semver');
var arrify = require('arrify');
var fs_1 = require('../utils/fs');
var path_1 = require('../utils/path');
exports.VALID_SOURCES = [
    'npm',
    'github',
    'bower',
    'ambient',
    'common'
];
function read(options) {
    var source = options.source, name = options.name;
    var path = path_1.normalizeSlashes(name) + ".json";
    var url = "https://raw.githubusercontent.com/typings/registry/master/" + source + "/" + path;
    invariant(exports.VALID_SOURCES.indexOf(source) > -1, "Source should be one of: " + exports.VALID_SOURCES.join(', '));
    return fs_1.readJsonFrom(url)
        .then(function (entry) {
        return select(entry, options);
    }, function (error) {
        if (error.type === 'EINVALIDSTATUS') {
            return Promise.reject(new TypeError(("Unable to find \"" + name + "\" from \"" + source + "\" in the registry. ") +
                "If you can contribute this typing, please help us out - https://github.com/typings/registry"));
        }
        return Promise.reject(error);
    });
}
exports.read = read;
function select(entry, options) {
    var version = options.version, name = options.name;
    var versions = Object.keys(entry.versions).sort(semver.compare);
    if (version == null) {
        return arrify(entry.versions[versions.pop()]);
    }
    var match;
    for (var _i = 0; _i < versions.length; _i++) {
        var value = versions[_i];
        if (semver.satisfies(value, version)) {
            match = value;
        }
    }
    if (match == null) {
        throw new TypeError("Unable to find \"" + name + "@" + version + "\". Available versions: " + versions.join(', '));
    }
    return arrify(entry.versions[match]);
}
exports.select = select;
function isRegistryPath(path) {
    return path.indexOf(':') === -1;
}
exports.isRegistryPath = isRegistryPath;
function parseRegistryPath(dep) {
    var _a = dep.split('@'), name = _a[0], version = _a[1];
    return { name: name, version: version };
}
exports.parseRegistryPath = parseRegistryPath;
//# sourceMappingURL=registry.js.map